@@ -15,26 +15,33 @@
 // We mean it.
 //
 
-#include <QtProtobuf/qtprotobufglobal.h>
-#include <QtCore/qtconfigmacros.h>
-#include <QtCore/qhash.h>
-
 #include "qprotobufmessage.h"
 
+#include <QtProtobuf/qtprotobufglobal.h>
+
+#include <QtCore/qtconfigmacros.h>
+#include <QtCore/qhash.h>
+#include <QtCore/private/qglobal_p.h>
+
+#include <optional>
+
 QT_BEGIN_NAMESPACE
 
+class QMetaProperty;
+
 class QProtobufMessagePrivate
 {
 public:
-    // QHash of form <bytes, occurrences>. A non-packed array is serialized as
-    // the (field index + wiretype) + bytes, which, if the serialized data is
-    // the same, will just be a bunch of identical segments, all of which we
-    // need.
-    QHash<QByteArray, qint32> unknownEntries;
+    // QHash of form <field index, data>.
+    QHash<qint32, QByteArrayList> unknownEntries;
     const QMetaObject *metaObject = nullptr;
 
     int getPropertyIndex(QAnyStringView propertyName) const;
-    void storeUnknownEntry(QByteArrayView entry);
+    void storeUnknownEntry(QByteArrayView entry, int fieldNumber);
+
+    std::optional<QMetaProperty> metaProperty(QAnyStringView name) const;
+    std::optional<QMetaProperty>
+    metaProperty(QtProtobufPrivate::QProtobufPropertyOrderingInfo ord) const;
 
     static QProtobufMessagePrivate *get(QProtobufMessage *message) { return message->d_func(); }
     static const QProtobufMessagePrivate *get(const QProtobufMessage *message)
